//device_lin.cpp: Linux usb functions using libusb
//Copyright 2006 Jeroen Domburg <s1fwx@jeroen.ietsmet.nl>
//This software is licensed under the GNU GPL.

#include "include.h"
#include "stdio.h"
#include "device.h"
#include <usb.h>

//Yuck, globals
static struct usb_dev_handle *device_handle;


// -----------------------------------------------------------------------------------------------
void device_set_cmd(LP_CMD cmd, CMD_ID cmd_id, uint16 ofs, uint32 size)
{
  memset(cmd, 0, sizeof(CMD));
  memcpy(cmd->id, CMD_HEADER_ID, 4);

  switch(cmd_id)
  {
  case CMD_INIT:
    cmd->cmd = _CMD_INIT;
    cmd->mode = CMD_MODE_READ;
    cmd->size = 0x0B;
    break;

  case CMD_ENTER_FMODE:
    cmd->cmd = _CMD_ENTER_FMODE;
    cmd->mode = CMD_MODE_READ;
    cmd->size = 0x01;
    break;

  case CMD_SHUTDOWN:
    cmd->cmd = _CMD_SHUTDOWN;
    break;

  case CMD_GET_SYSINFO:    
    cmd->cmd = _CMD_READ_SYSINFO(0x0800);
    cmd->mode = CMD_MODE_READ;
    cmd->size = 0xC0;
    cmd->unkwn0x13 = 0xC008C000;
    cmd->unkwn0x17 = 0x43BC2800;
    break;

  case CMD_READ_BREC:
    cmd->mode = CMD_MODE_READ;
  case CMD_WRITE_BREC:
    cmd->cmd = _CMD_RWX_BREC(ofs);
    cmd->size = 0x4000;
    cmd->unkwn0x13 = (size>>9)<<24; //0x20070000;
    //cmd->unkwn0x17 = 0x0040000;
    break;

  case CMD_READ_FLASH:
    cmd->mode = CMD_MODE_READ;
    cmd->cmd = _CMD_READ_FLASH(ofs);
    cmd->size = size;
    cmd->unkwn0x13 = (size>>9)<<24;
    //cmd->unkwn0x17 = 0x107D4800;
    break;

  case CMD_WRITE_FWARE:
    cmd->cmd = _CMD_WRITE_FWARE(ofs);
    cmd->size = size;
    cmd->unkwn0x13 = (size>>9)<<24;
    cmd->unkwn0x17 = 0x107D4800;
    break;
  }
}




// -----------------------------------------------------------------------------------------------
HANDLE device_open(char drive)
{
   	struct usb_bus *busses;
    	struct usb_bus *bus;
	bool found=false;
	int r;
    
    	usb_init();
    	usb_find_busses();
    	usb_find_devices();
    

    	busses = usb_get_busses();
    	for (bus = busses; bus; bus = bus->next) {
    		struct usb_device *dev;
    
    		for (dev = bus->devices; dev; dev = dev->next) {
			if ((dev->descriptor.idVendor==4310)&&(dev->descriptor.idProduct==65361) && (!found)) {
			    device_handle=usb_open(dev);
			    r=usb_claim_interface(device_handle, 0);
			    if (r<0) perror("Claim if");
			    found=true;
			    printf("device_lin: device found\n");
			}
		}
    	}
	//check device_handle for error... nah
    return found;
}


// -----------------------------------------------------------------------------------------------
bool device_close(HANDLE *hd)
{
  usb_close(device_handle);
  return true;
}


// -----------------------------------------------------------------------------------------------
long device_send(HANDLE hd, void *data, uint32 max_data)
{
  return usb_bulk_write(device_handle, 1, (char *)data, max_data, 1024);
}


static void printbuff(char* buff, int len) {
    int x;
    for (x=0; x<len; x++) {
	printf("%02hhx ",buff[x]);
	if ((x&15)==15) printf("\n");
    }
    printf("\n");
}

// -----------------------------------------------------------------------------------------------
long device_send_cmd(HANDLE hd, LP_CMD cmd, void *data, uint32 max_data)
{
  uint8 tx_buf[CMD_BUFSIZE];
  uint8 csw[CMD_BUFSIZE];
  uint32 size;
  char *c;
  int r;
  
  fprintf(stderr,"Send cmd 0x%x\n",cmd->cmd);
  //refill cmd
  cmd->size=max_data;
  cmd->undef0x04=0xEFBEADDE; //0xDEADBEEF as tag, gets noticed in USB dumps :)
  //Standard CBW header
  cmd->id[0]='U';cmd->id[1]='S';
  cmd->id[2]='B';cmd->id[3]='C';
  
  //Write the data
  fprintf(stderr,"Write cmd, 0x1f bytes.\n");
  printbuff((char*)cmd,0x1f);
  r=usb_bulk_write(device_handle, 1, (char*)cmd, 0x1f, 100);
  if (r<0) {
    perror("Error @ write cbw");
    return(-1);
  }
    
  if (max_data>0) {
    if(cmd->mode == 0x80) {   // read data...
      fprintf(stderr,"Read data, 0x%x bytes\n",max_data);
      r=usb_bulk_read(device_handle, 0x82, (char*)data, max_data, 1000);
      if (r<0) {
        perror("Read data:"); 
	return(-1);
      }
      size=r;
    } else {                  // send data...
      fprintf(stderr,"Write data, 0x%x bytes\n",max_data);
      r=usb_bulk_write(device_handle, 1, (char*)data,  max_data, 1000);
      if (r<0) {
        perror("Write data:");
	return(-1);
      }
    }
    fprintf(stderr,"Done -- 0x%x bytes.\n",r);
  }
  //Read csw
  fprintf(stderr,"Read csw\n");
  r=usb_bulk_read(device_handle, 0x82, (char*)tx_buf, 0x0d, 100);
    printf("Got 0x%x bytes.\n",r);
  fprintf(stderr,"csw=%s\n",tx_buf);
  if (r<0) {
    perror("Error at read csw:");
    return(-1);
  }
  return (long)size;
}


// -----------------------------------------------------------------------------------------------
long device_send_cmd(HANDLE hd, CMD_ID cmd_id, void *data, uint32 max_data)
{
  CMD cmd;
  device_set_cmd(&cmd, cmd_id);
  return device_send_cmd(hd, &cmd, data, max_data);
}


bool device_detect(char drive) {
    return true;
}



// -----------------------------------------------------------------------------------------------
// returns bit mask
// bit=0 => not detected, bit=1 => detected
// offset 0 => A:, 1 => B:, 2 => C:, ... 
DWORD device_scan()
{
//ToDo: Create a true Linuxy interface.
//Right now this just returns A as an available drive and checks if this is true later.
  return 1;
}

